/*
 REACTIVE AUTONOMOUS BLACKHOLE LIST SERVER
 COPYRIGHT (C) 2005 DEEP LOGIC, INC.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/* btree: binary tree index */

#ifndef _BTREE_H
#define _BTREE_H

enum
{ btree_num_primes = 28 };

/* btree root */
struct btree
{
  unsigned long items;
  struct btree_node *root;
  pthread_rwlock_t lock;
};

/* btree node (a single long hashed token) */
struct btree_node
{
  unsigned long key;
  void *ptr;
  struct btree_node *left;
  struct btree_node *right;
  struct btree_node *parent;
};

/* constructor and destructor */
struct btree *btree_create (void);
int btree_destroy (struct btree *btree);

/* public functions */
struct btree_node *btree_first (struct btree *btree);
struct btree_node *btree_next (struct btree_node *node);
struct btree_node *btree_find (struct btree *btree, unsigned long key);
struct btree_node *btree_find_parent (struct btree *btree, unsigned long key);

int btree_add (struct btree *btree, unsigned long key, void *ptr);
int btree_del (struct btree *btree, unsigned long key);

#endif /* _BTREE_H */
