/*
 REACTIVE AUTONOMOUS BLACKHOLE LIST SERVER
 COPYRIGHT (C) 2005 DEEP LOGIC INC

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifdef HAVE_CONFIG_H
#include <auto-config.h>
#endif

#ifndef _RABL_SERVER_H
#  define _RABL_SERVER_H

#define	SYNTAX	"Syntax: rabl_server [-c config] [-u uid]\n"

#define PROTOCOL_VERSION	"1"	/* Protocol Version 1 */
#define CONNECTION_TIMEOUT	3	/* In Seconds */
#define MAX_CONN		10	/* Maximum Connections per IP*/

#define SH_PACKET_MAX	1024

char *config_file;
fd_set master;
fd_set read_fds;

struct rabl_server_config {
  struct btree *auth;		/* Sorted by uid */
  struct btree *peers;		/* Sorted by server id */
  struct btree *rabl_ipv4;	/* Sorted by 32-bit address */
  struct btree *connections;	/* Sorted by file descriptor */
  struct hash *conn_count;	/* Connection Counts per IP */
  struct hash *rabl_ipv6;	/* Hashed in plain text */
  int port;		/* Bind Port */
  int queue;		/* Connection Queue Length */
  int cleanup;		/* Cleanup Interval */
  int minspread;	/* Minimum Spread to get blacklisted */
  unsigned long life;	/* Blacklist life */
  char *log;		/* Logfile */
  char *doscmd;		/* DoS Cmd */
  char *zone_template;
  char *zone_output;
  pthread_rwlock_t lock;	/* Lock for reloading */
};

struct rabl_auth {
  unsigned long uid;
  char *shared_secret;
  char *acl;
  int privileges;	/* AUTH_ */
  int failed_attempts;
  time_t last_failed_attempt;
  time_t locked_since;
};

struct rabl_data_ipv4 {
  char type;
  uint32_t address;
};

struct rabl_data_ipv6 {
  char type;
  char address[39];
};

/* Server Data */

struct rabl_data_sipv4 {
  char type;
  uint32_t address;
  uint32_t uid;
};

struct rabl_data_sipv6 {
  char type;
  char address[39];
  uint32_t uid;
};

struct rabl_data_auth {
  uint32_t uid;
};

struct rabl_address {
  struct btree *reporting_networks;
};

struct rabl_network {
  unsigned long spam_hits;
  time_t last_hit;
};

typedef struct {
  int sockfd;                   /* Socket fd */
  int invalid_requests;         /* Invalid Requests */
  int packet_len;               /* Length of packet after header recv */
  char packet_type;		/* Packet Type being sent */
  int recv_len;                 /* Bytes received thus far */
  time_t last_receive;          /* Last successful recv() */
  time_t last_full_receive;     /* Last recv() of full packet */
  char data[SH_PACKET_MAX];
  struct rabl_auth *auth;		/* User, if authenticated */
  struct sockaddr_in remote_addr;
  pthread_t thread;
} connection;

struct thread_process {
  connection *conn;
};

struct unique_address {
  int count;
};

struct rabl_server_config *config = NULL;

/* Functions */
void	read_config(int);
void	reload_config(int);
int 	destroy_user_config(void);
int	destroy_config(void);
int	daemon_listen(void);
void *	object_cleanup(void *);
void *	process_connection(void *);
int	process_ipv4_packet(connection *conn);
int	process_ipv6_packet(connection *conn);
int	process_auth_packet(connection *conn);
int	process_sipv4_packet(connection *conn);
int	process_sipv6_packet(connection *conn);
int	end_connection(connection *conn);
int	receive_sock(int sockfd, void *buff, int len);
int	check_incoming(struct sockaddr_in *remote_addr);
int	free_incoming(struct sockaddr_in *remote_addr);
void	dieout(int sig);

#define AUTH_NONE	0x00
#define AUTH_RDWR	0x01
#define AUTH_SERVER	0x02
#define AUTH_LOCKED	0xFF

#define DATA_QUERY		0x00
#define DATA_POST_SPAM		0x01
#define DATA_POST_HAM		0x02

#define PT_IPV4		0x01
#define PT_IPV6 	0x02
#define	PT_AUTH		0x03
#define PT_SIPV4	0x04
#define PT_SIPV6	0x05

#ifndef EINVAL
#       define EINVAL                   -0x01
  /* Invalid call or parms - Already Initialized or shutdown */
#endif
#define EUNKNOWN                -0x02   /* Unknown/unexpected error */
#define EFILE                   -0x03   /* File error */
#define ELOCK                   -0x04   /* Lock error */
#define EFAILURE                -0x05   /* Failed to perform operation */

#endif /* _RABL_SERVER_H */

